using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class SfxManagerBase<T> : SingletonMonoBehaviour<T> where T : SfxManagerBase<T>
{
    [SerializeField] private float globalVolumeMultiplier = 0.1f;
    [SerializeField] private float defaultSoundCooldown = 0.1f;
    
    private AudioListener audioListener;

    private List<AudioSource> createdAudioSources;

    private Settings settings;

    private List<AudioClip> clipsInSoundCooldowns;
    private Dictionary<AudioClip, float> soundCooldowns;

    public bool Muted
    {
        get { return (settings != null) && (!settings.Sfx); }
    }

    protected virtual void Awake()
    {
        clipsInSoundCooldowns = new List<AudioClip>();
        soundCooldowns = new Dictionary<AudioClip, float>();
        createdAudioSources = new List<AudioSource>();

        settings = Settings.Instance;

        if (settings != null)
            settings.EventSfxChanged += OnSfxVolumeChanged;

        UpdateMuteSettings();
    }

    private void Start()
    {
        audioListener = UnityHelper.FindFirstActiveInstance<AudioListener>();
    }

    private void OnDestroy()
    {
        if (settings != null)
            settings.EventSfxChanged -= OnSfxVolumeChanged;
    }

    private void Update()
    {
        foreach (var clip in clipsInSoundCooldowns)
        {
            soundCooldowns[clip] -= Time.deltaTime;
        }
    }

    private void OnSfxVolumeChanged(bool sfxActive)
    {
        UpdateMuteSettings();
    }

    private void UpdateMuteSettings()
    {
        foreach (var audioSource in createdAudioSources)
        {
            audioSource.mute = Muted;
        }

        var audioSources = (AudioSource[]) FindObjectsOfType(typeof (AudioSource));
        foreach (var audioSource in audioSources)
        {
            if (audioSource.gameObject.name == "One shot audio")
            {
                audioSource.mute = Muted;
            }
        }
    }

    public bool IsAudioSourcePlaying(AudioSource audioSource)
    {
        return (audioSource != null) && audioSource.isPlaying;
    }

    public void SetAudioSourcePlaying(AudioSource audioSource, bool value)
    {
        if (audioSource == null)
            return;

        if (audioSource.isPlaying == value)
            return;

        if (value)
        {
            audioSource.Play();
        }
        else
        {
            audioSource.Pause();
        }
    }

    public void PlaySound(AudioClipWithVolume[] audioClips)
    {
        if ((audioClips == null) || (audioClips.Length == 0))
            return;

        var audioClip = audioClips[UnityEngine.Random.Range(0, audioClips.Length)];
        PlaySound(audioClip);
    }

    public void PlaySound(AudioClipWithVolume audioClip)
    {
        if (audioClip == null)
            return;

        PlaySound(audioClip.Clip, audioClip.Volume);
    }

    public void PlaySound(AudioClip[] audioClips, float volume = 1f)
    {
        if ((audioClips == null) || (audioClips.Length == 0))
            return;

        var audioClip = audioClips[UnityEngine.Random.Range(0, audioClips.Length)];

        PlaySound(audioClip, volume);
    }

    public void PlaySound(AudioClip audioClip, float volume = 1f, bool useGlobalVolumeMultiplier = true)
    {
        if (audioListener == null)
            return;

        if (Muted)
            return;

        if (audioClip == null)
            return;

        if (defaultSoundCooldown > 0)
        {
            if (clipsInSoundCooldowns.Contains(audioClip))
            {
                if (soundCooldowns[audioClip] > 0)
                    return;
            }
            else
            {
                clipsInSoundCooldowns.Add(audioClip);
            }

            soundCooldowns[audioClip] = defaultSoundCooldown;
        }


        var position = audioListener.transform.position;

        if (useGlobalVolumeMultiplier)
            volume *= globalVolumeMultiplier;

        AudioSource.PlayClipAtPoint(audioClip, position, volume);
    }

    public AudioSource CreateAudioSource(AudioClipWithVolume clip)
    {
        var audioSourceName = "Unassigned Audio Source";
        if ((clip != null) && (clip.Clip != null))
            audioSourceName = "Audio Source: " + clip.Clip.name;

        var newGameObject = new GameObject(audioSourceName);
        newGameObject.transform.parent = transform;

        var audioSource = newGameObject.AddComponent<AudioSource>();

        /*
        if (!clip.IgnoreGlobalVolumeMultiplier)
        {
            audioSource.volume *= globalVolumeMultiplier;
        }
         */

        if (clip != null)
        {
            audioSource.volume = clip.Volume * globalVolumeMultiplier;
            audioSource.clip = clip.Clip;
            audioSource.loop = true;
            audioSource.mute = Muted;
        }

        createdAudioSources.Add(audioSource);

        return audioSource;
    }
}
