using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class GuiButton : MonoBehaviour
{
    public delegate void ButtonDelegate(GuiButton button);
    public delegate void ButtonRectChangedDelegate(GuiButton button);

    private GuiManager guiManager;
    private VisualSettings visualSettings;
    private SfxManager sfxManager;
    private ScreenManager screenManager;

    private string text;
    private Vector2 positionAbsolute;
    private float positionRelativeWidthX;
    private Vector2 positionRelative;
    private Vector2 relativeAnchor;
    private Color color;
    private Font font;
    private float fontSize;
    private bool _hovering;
    private bool _marked;

    public Rect ScreenRect { get; private set; }
    public Rect GuiRect { get; private set; }

    public event ButtonDelegate EventButtonClicked;
    public event ButtonDelegate EventButtonHoverStarted;
    public event ButtonDelegate EventButtonHoverEnded;
    public event ButtonRectChangedDelegate EventButtonRectUpdated;

    public bool Draw { get; set; }

    private bool _inArea;
    private bool initialized;
    private bool _activated;

    private void Awake()
    {
        guiManager = GuiManager.Instance;
        visualSettings = VisualSettings.Instance;
        sfxManager = SfxManager.Instance;
        screenManager = ScreenManager.Instance;

        font = visualSettings.FontDefault;

        _activated = true;

        Draw = true;
    }

    private void OnEnable()
    {
        RefreshRect();

        screenManager.EventScreenSizeChanged += OnScreenSizeChanged;
    }

    private void OnDisable()
    {
        screenManager.EventScreenSizeChanged -= OnScreenSizeChanged;
        InArea = false;
    }

    private void OnDestroy()
    {
    }

    public void InitializeText(string text, float positionRelativeWidthX, Vector2 positionRelative, Vector2 relativeAnchor, float fontSize, ButtonDelegate clickedEvent = null)
    {
        initialized = true;

        this.text = "[" + text + "]";
        this.positionRelativeWidthX = positionRelativeWidthX;
        this.positionRelative = positionRelative;
        this.relativeAnchor = relativeAnchor;

        this.fontSize = fontSize;

        if (clickedEvent != null)
            EventButtonClicked += clickedEvent;

        UpdateColor();
        RefreshRect();
    }

    private void Update()
    {
        if (!Activated || !Draw)
            return;

        var mousePosition = Input.mousePosition;

        InArea = ScreenRect.Contains(mousePosition);
        if (InArea && Input.GetMouseButtonDown(0))
        {
            Clicked();
        }
    }

    private void OnGUI()
    {
        if (!Draw)
            return;

        UnityHelper.SetFont(font, fontSize);
        UnityHelper.ShowText(text, PositionAbsolute + new Vector2(PositionRelativeWidthX * Screen.width, 0f) + PositionRelative * Screen.height, relativeAnchor, color);
    }

    private void Clicked()
    {
        if (!Activated)
            return;

        sfxManager.PlayButtonClicked();

        if (EventButtonClicked != null)
            EventButtonClicked(this);
    }

    public bool InArea
    {
        get { return _inArea; }
        set
        {
            if (_inArea == value)
                return;

            _inArea = value;

            guiManager.MouseOverButtonAreaChange(this, _inArea);
        }
    }

    public bool Hovering
    {
        get { return _hovering; }
        set
        {
            if (_hovering == value)
                return;

            _hovering = value;
            UpdateColor();

            if (_hovering)
            {
                sfxManager.PlayButtonHovered();

                if (EventButtonHoverStarted != null)
                    EventButtonHoverStarted(this);
            }
            else
            {
                if (EventButtonHoverEnded != null)
                    EventButtonHoverEnded(this);
            }
        }
    }

    private void OnScreenSizeChanged()
    {
        RefreshRect();
    }

    public Vector2 PositionAbsolute
    {
        get { return positionAbsolute; }
        set
        {
            if (positionAbsolute == value)
                return;

            positionAbsolute = value;
            RefreshRect();
        }
    }

    public float PositionRelativeWidthX
    {
        get { return positionRelativeWidthX; }
        set
        {
            if (positionRelativeWidthX == value)
                return;

            positionRelativeWidthX = value;
            RefreshRect();
        }
    }

    public Vector2 PositionRelative
    {
        get { return positionRelative; }
        set
        {
            if (positionRelative == value)
                return;

            positionRelative = value;
            RefreshRect();
        }
    }

    public bool Marked
    {
        get { return _marked; }
        set
        {
            if (_marked == value)
                return;

            _marked = value;
            UpdateColor();
        }
    }

    public bool Activated
    {
        get { return _activated; }
        set
        {
            if (_activated == value)
                return;

            _activated = value;
            UpdateColor();

            if (!_activated)
                InArea = false;
        }
    }

    private void RefreshRect()
    {
        if (!initialized)
            return;

        var newRect = UnityHelper.GetTextRect(font, fontSize, text, PositionAbsolute + new Vector2(PositionRelativeWidthX * Screen.width, 0f) + PositionRelative * Screen.height, relativeAnchor);
        if (newRect == GuiRect)
            return;

        GuiRect = newRect;

        var rect = GuiRect;
        rect.y = Screen.height - rect.y - rect.height;
        ScreenRect = rect;

        if (EventButtonRectUpdated != null)
            EventButtonRectUpdated(this);
    }

    private void UpdateColor()
    {
        if (!Activated)
        {
            color = visualSettings.ButtonColorDeactivated;
        }
        else if (Hovering)
        {
            color = visualSettings.ButtonColorHovered;
        }
        else
        {
            color = Marked ? visualSettings.ButtonColorMarked : visualSettings.ButtonColorNormal;
        }
    }
}
