﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class GuiManagerBase<T> : SingletonMonoBehaviour<T> where T : GuiManagerBase<T>
{
    protected VisualSettings visualSettings;
    protected Camera mainCamera;

    protected List<GuiVanishingText> texts;

    private List<GuiButton> mouseOverButtons;
    private GuiButton _currentlyHoveredButton;

    public Font Font { get { return visualSettings.FontDefault; } }

    public bool Stopped { get; set; }

    protected virtual void Awake()
    {
        visualSettings = VisualSettings.Instance;
        mainCamera = Camera.main;

        texts = new List<GuiVanishingText>();

        mouseOverButtons = new List<GuiButton>();
    }

    protected virtual void Update()
    {
        if (mouseOverButtons.Count > 1)
        {
            UpdateHoveredMouseButtonAmongstMultiple();
        }

        if (Stopped)
            return;

        foreach (var text in texts)
        {
            text.Update();
        }

        texts.RemoveAll(text => text.Vanished);
    }

    protected virtual void OnGUI()
    {
        if (Stopped)
            return;

        foreach (var text in texts)
        {
            text.Draw();
        }
    }

    public Vector2 GetScreenPosition(Vector2 worldPosition)
    {
        var screenPosition = mainCamera.WorldToScreenPoint(worldPosition);
        screenPosition.y = Screen.height - screenPosition.y;
        return screenPosition;
    }

    public void Clear()
    {
        texts.Clear();
    }

    public void MouseOverButtonAreaChange(GuiButton guiButton, bool inArea)
    {
        if (inArea)
        {
            mouseOverButtons.Add(guiButton);
        }
        else
        {
            mouseOverButtons.Remove(guiButton);
        }

        if (mouseOverButtons.Count == 0)
        {
            CurrentlyHoveredButton = null;
        }
        else if (mouseOverButtons.Count == 1)
        {
            CurrentlyHoveredButton = mouseOverButtons[0];
        }
        else
        {
            UpdateHoveredMouseButtonAmongstMultiple();
        }
    }

    private void UpdateHoveredMouseButtonAmongstMultiple()
    {
        Vector2 mousePosition = Input.mousePosition;
        CurrentlyHoveredButton = mouseOverButtons.OrderBy(button => Vector2.Distance(button.ScreenRect.center, mousePosition)).FirstOrDefault();
    }

    private GuiButton CurrentlyHoveredButton
    {
        get { return _currentlyHoveredButton; }
        set
        {
            if (_currentlyHoveredButton == value)
                return;

            if (_currentlyHoveredButton != null)
                _currentlyHoveredButton.Hovering = false;

            _currentlyHoveredButton = value;

            if (_currentlyHoveredButton != null)
                _currentlyHoveredButton.Hovering = true;
        }
    }
}
