using UnityEngine;

public class SlowMotion : SingletonMonoBehaviour<SlowMotion>
{
    private enum Status
    {
        SlowingDown,
        SpeedingUp,
        TargetReached
    }

    [SerializeField] private float slowMotionTimeScale = 0.1f;
    [SerializeField] private float speedSlowDown = 10f;
    [SerializeField] private float speedSpeedUp = 10f;
    [SerializeField] private bool adjustFixedTimeStep = true;
    [SerializeField] private float lowestPossibleFixedTimeStep = 0.001f;

    private float defaultFixedDeltaTime;
    private Countdown slowDownCountdown;
    private Status status;

    private void Awake()
    {
        status = Status.TargetReached;
        defaultFixedDeltaTime = Time.fixedDeltaTime;
    }

    private void Start()
    {
        slowDownCountdown = new Countdown(0f);
    }

    private void FixedUpdate()
    {
        if (status == Status.SlowingDown)
        {
            MoveTimeScaleTowardsTarget(slowMotionTimeScale, speedSlowDown, UnityHelper.RealDeltaTime);
        }
        else if (status == Status.SpeedingUp)
        {
            MoveTimeScaleTowardsTarget(1f, speedSpeedUp, UnityHelper.RealDeltaTime);
        }

        if (!slowDownCountdown.ReachedZero)
        {
            if (slowDownCountdown.Update())
            {
                status = Status.SpeedingUp;
            }
        }
    }

    private void MoveTimeScaleTowardsTarget(float target, float speed, float realTimeDelta)
    {
        var newTimeScale = (speed == 0)
                            ? target
                            : Mathf.MoveTowards(Time.timeScale, target, speed * realTimeDelta);

        SetTimeScale(newTimeScale);

        if (Time.timeScale == target)
        {
            status = Status.TargetReached;
        }
    }

    public void StartSlowMotion(float duration)
    {
        if (slowDownCountdown.TimeLeft > duration)
            return;

        slowDownCountdown.Reset(duration);
        status = Status.SlowingDown;
    }

    public void StopSlowMotion()
    {
        if (Time.timeScale == 1f)
            return;

        slowDownCountdown.Reset(0f);
        status = Status.SpeedingUp;
    }

    public void SetTimeScale(float value)
    {
        Time.timeScale = value;

        if (adjustFixedTimeStep)
            Time.fixedDeltaTime = Mathf.Max(defaultFixedDeltaTime * Time.timeScale, lowestPossibleFixedTimeStep);
    }
}
