﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;
using XInputDotNetPure;

public class GamepadInputXInput : GamepadInput
{
    [SerializeField] private int gamepadCount = 2;
    [SerializeField] private bool axisPrimaryYReversed = false;
    [SerializeField] private bool axisSecordaryYReversed = false;

    private class GamepadData
    {
        public PlayerIndex PlayerIndex { get; private set; }

        public GamePadState CurrentState { get; private set; }
        public GamePadState PreviousState { get; private set; }

        private float currentVibrationLeft;
        private float currentVibrationRight;

        private float newVibrationLeft;
        private float newVibrationRight;

        public GamepadData(PlayerIndex playerIndex)
        {
            PlayerIndex = playerIndex;

            newVibrationLeft = -1;
            newVibrationRight = -1;
        }

        public void Update()
        {
            PreviousState = CurrentState;
            CurrentState = GamePad.GetState(PlayerIndex);
        }

        public void LateUpdate()
        {
            if ((newVibrationLeft < 0) && (newVibrationRight < 0))
                return;

            if (newVibrationLeft < 0)
                newVibrationLeft = currentVibrationLeft;

            if (newVibrationRight < 0)
                newVibrationRight = currentVibrationRight;

            if ((newVibrationLeft == currentVibrationLeft) && (newVibrationRight == currentVibrationRight))
            {
                newVibrationLeft = -1;
                newVibrationRight = -1;
                return;
            }

            GamePad.SetVibration(PlayerIndex, newVibrationLeft, newVibrationRight);
            currentVibrationLeft = newVibrationLeft;
            currentVibrationRight = newVibrationRight;

            newVibrationLeft = -1;
            newVibrationRight = -1;
        }

        public void SetLowestVibrationThisFrame(float left, float right)
        {
            newVibrationLeft = Mathf.Max(newVibrationLeft, left);
            newVibrationRight = Mathf.Max(newVibrationRight, right);
        }

        public void ClearVibration()
        {
            GamePad.SetVibration(PlayerIndex, 0, 0);
            currentVibrationLeft = 0;
            currentVibrationRight = 0;
            newVibrationLeft = -1;
            newVibrationRight = -1;
        }
    }

    private GamepadData[] dataList;

    private void Awake()
    {
        dataList = new GamepadData[gamepadCount];
        for (int i = 0; i < gamepadCount; i++)
        {
            dataList[i] = new GamepadData((PlayerIndex) i);
        }
    }

    private void OnDestroy()
    {
        foreach (var data in dataList)
        {
            data.ClearVibration();
        }
    }

    private void Update()
    {
        foreach (var data in dataList)
        {
            data.Update();
        }
    }

    private void LateUpdate()
    {
        foreach (var data in dataList)
        {
            data.LateUpdate();
        }
    }

    public override bool GetButton(int gamepadIndex, Button button)
    {
        var data = dataList[gamepadIndex - 1];
        return IsButtonPressed(data.CurrentState, button);
    }

    public override bool GetButtonDown(int gamepadIndex, Button button)
    {
        var data = dataList[gamepadIndex - 1];
        return !IsButtonPressed(data.PreviousState, button) && IsButtonPressed(data.CurrentState, button);
    }

    public override bool GetButtonUp(int gamepadIndex, Button button)
    {
        var data = dataList[gamepadIndex - 1];
        return IsButtonPressed(data.PreviousState, button) && !IsButtonPressed(data.CurrentState, button);
    }

    public override bool GetButton(Button button)
    {
        return dataList.Any(data => IsButtonPressed(data.CurrentState, button));
    }

    public override bool GetButtonDown(Button button)
    {
        for (int gamepadIndex = 1; gamepadIndex <= gamepadCount; gamepadIndex++)
        {
            if (GetButtonDown(gamepadIndex, button))
                return true;
        }

        return false;
    }

    public override bool GetButtonUp(Button button)
    {
        for (int gamepadIndex = 1; gamepadIndex <= gamepadCount; gamepadIndex++)
        {
            if (GetButtonUp(gamepadIndex, button))
                return true;
        }

        return false;
    }

    public override float GetAxis(int gamepadIndex, Axis axis)
    {
        var state = dataList[gamepadIndex - 1].CurrentState;

        switch (axis)
        {
            case Axis.PrimaryX:
                return state.ThumbSticks.Left.X;

            case Axis.PrimaryY:
                return state.ThumbSticks.Left.Y * (axisPrimaryYReversed ? -1 : 1);

            case Axis.SecondaryX:
                return state.ThumbSticks.Right.X;

            case Axis.SecondaryY:
                return state.ThumbSticks.Right.Y * (axisSecordaryYReversed ? -1 : 1);

            case Axis.Trigger:
                return state.Triggers.Right - state.Triggers.Left;

            default:
                throw new NotImplementedException(axis.ToString());
        }
    }

    private bool IsButtonPressed(GamePadState state, Button button)
    {
        switch (button)
        {
            case Button.A:
                return state.Buttons.A == ButtonState.Pressed;

            case Button.B:
                return state.Buttons.B == ButtonState.Pressed;

            case Button.X:
                return state.Buttons.X == ButtonState.Pressed;

            case Button.Y:
                return state.Buttons.Y == ButtonState.Pressed;

            case Button.Start:
                return state.Buttons.Start == ButtonState.Pressed;

            case Button.Back:
                return state.Buttons.Back == ButtonState.Pressed;

            case Button.LeftBack:
                return state.Buttons.LeftShoulder == ButtonState.Pressed;

            case Button.RightBack:
                return state.Buttons.RightShoulder == ButtonState.Pressed;

            default:
                throw new NotImplementedException(button.ToString());
        }
    }

    public override void SetLowestVibrationThisFrame(int gamepadIndex, float left, float right)
    {
        dataList[gamepadIndex - 1].SetLowestVibrationThisFrame(left, right);
    }
}
