﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts.Pathfinding
{
    // https://en.wikipedia.org/wiki/A*_search_algorithm#Pseudocode
    public class AStar
    {
        public class Node
        {
            public float GScore { get; set; }
            public float FScore { get; set; }
            public Vector2 Position { get; private set; }
            public Node CameFrom { get; set; }

            public Node(Vector2 position)
            {
                Position = position;
            }
        }

        private List<Node> nodes;
        private Dictionary<Node, HashSet<Node>> connections;

        public AStar()
        {
            nodes = new List<Node>();
            connections = new Dictionary<Node, HashSet<Node>>();
        }

        public List<Vector2> SeekPath(Node start, Node goal)
        {
            var closedSet = new HashSet<Node>();
            var openSet = new List<Node>();
            //var cameFrom = new Dictionary<Node, Node>();

            start.CameFrom = null;
            start.GScore = 0;
            start.FScore = start.GScore + HeuristicCostEstimate(start, goal);

            openSet.Add(start);

            while (openSet.Count > 0)
            {
                var current = openSet.First();
                if (current == goal)
                    return ReconstructPath(goal);

                openSet.RemoveAt(0);
                closedSet.Add(current);

                foreach (var neighbor in NeighborNodes(current))
                {
                    if (closedSet.Contains(neighbor))
                        continue;

                    var tentativeGScore = current.GScore + HeuristicCostEstimate(current, neighbor);

                    if (!openSet.Contains(neighbor) || (tentativeGScore < neighbor.GScore))
                    {
                        //cameFrom[neighbor] = current;
                        neighbor.CameFrom = current;
                        neighbor.GScore = tentativeGScore;
                        neighbor.FScore = neighbor.GScore + HeuristicCostEstimate(neighbor, goal);

                        openSet.Add(neighbor);
                        openSet.Sort((a, b) => a.GScore.CompareTo(b.GScore));
                    }
                }
            }

            return null;
        }

        public void AddConnection(Vector2 from, Vector2 to, bool bidirectional)
        {
            var fromNode = GetOrCreateNode(from);
            var toNode = GetOrCreateNode(to);

            connections[fromNode].Add(toNode);
            if (bidirectional)
                connections[toNode].Add(fromNode);
        }

        private Node GetOrCreateNode(Vector2 position)
        {
            var node = nodes.FirstOrDefault(n => n.Position == position);
            if (node == null)
            {
                node = new Node(position);
                nodes.Add(node);

                connections[node] = new HashSet<Node>();
            }

            return node;
        }

        private List<Vector2> ReconstructPath(Node currentNode)
        {
            var path = new List<Vector2>();
            while (currentNode != null)
            {
                path.Insert(0, currentNode.Position);
                currentNode = currentNode.CameFrom;
            }
            return path;
        }

        public IEnumerable<Node> Nodes { get { return nodes; } } 

        private IEnumerable<Node> NeighborNodes(Node node)
        {
            return connections[node];
        }

        private float HeuristicCostEstimate(Node start, Node goal)
        {
            return Vector2.Distance(start.Position, goal.Position);
        }

        public void GizmoDrawConnections()
        {
            foreach (var connectionsFromTo in connections)
            {
                foreach (var connectionsTo in connectionsFromTo.Value)
                {
                    Gizmos.DrawLine(connectionsFromTo.Key.Position, connectionsTo.Position);
                    Gizmos.DrawCube(connectionsFromTo.Key.Position, Vector3.one * 0.1f);
                    Gizmos.DrawCube(connectionsTo.Position, Vector3.one * 0.1f);
                }
            }
        }
    }
}
