﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Xml;
using UnityEngine;

public class LeaderboardManager : SingletonMonoBehaviour<LeaderboardManager>
{
    public class ScoreEntry
    {
        public int Placing { get; private set; }
        public string Username { get; private set; }
        public int Score { get; private set; }
        public DateTime Date { get; private set; }

        public ScoreEntry(int placing, string username, int score, DateTime date)
        {
            Placing = placing;
            Username = username;
            Score = score;
        }
    }

    public delegate void StatusCallbackSector(bool success, int sectorIndex);
    public delegate void StatusCallbackLevel(bool success, int levelScoreId);
    public delegate void StatusCallbackPlayer(bool success, string username);
    public delegate void ScoreSectorCallback(int sectorIndex, List<ScoreEntry> scores);
    public delegate void ScoreLevelCallback(int levelScoreId, List<ScoreEntry> scores);

    [SerializeField] private bool submitScoresInEditor = false;

    private ScoreoidManager scoreoidManager;

    private void Awake()
    {
        scoreoidManager = ScoreoidManager.Instance;
    }

    public void CreatePlayerIfNotExists(string username, StatusCallbackPlayer statusCallback)
    {
        CreatePlayer(username, statusCallback);
    }

    private void CreatePlayer(string username, StatusCallbackPlayer statusCallback)
    {
        scoreoidManager.SendRequest("createPlayer",
                                    delegate(bool success, string result, string error)
                                        {
                                            if (statusCallback != null)
                                                statusCallback(success, username);
                                        },
                                    "username", username);
    }

    /*
    private void ConfirmPlayerExistence(string username, StatusCallbackPlayer statusCallback)
    {
        throw new NotImplementedException("Actually check player existence");

        var form = scoreoidManager.CreateForm();
        form.AddField("response", "xml");
        form.AddField("username", username);

        scoreoidManager.SendRequest("getPlayer", form, delegate(bool success, string result, string error)
                                                              {
                                                                  if (statusCallback != null)
                                                                      statusCallback(success, username);
                                                              });
    }
     */

    public void SubmitScoreSector(int sectorIndex, string username, int score, StatusCallbackSector statusCallback = null)
    {
        if (Application.isEditor && !submitScoresInEditor)
        {
            if (statusCallback != null)
                statusCallback(true, sectorIndex);

            return;
        }

        scoreoidManager.SendRequest("createScore",
                                    delegate(bool success, string result, string error)
                                        {
                                            if (statusCallback != null)
                                                statusCallback(success, sectorIndex);
                                        },
                                    "response", "xml",
                                    "difficulty", SectorIndexToDifficultyIndex(sectorIndex).ToString(),
                                    "username", username,
                                    "score", score.ToString());
    }

    public void SubmitScoreLevel(int levelScoreId, string username, int score, StatusCallbackLevel statusCallback = null)
    {
        if (Application.isEditor && !submitScoresInEditor)
        {
            if (statusCallback != null)
                statusCallback(true, levelScoreId);

            return;
        }

        scoreoidManager.SendRequest("createScore",
                                    delegate(bool success, string result, string error)
                                        {
                                            if (statusCallback != null)
                                                statusCallback(success, levelScoreId);
                                        },
                                    "response", "xml",
                                    "difficulty", levelScoreId.ToString(CultureInfo.InvariantCulture),
                                    "username", username,
                                    "score", score.ToString(CultureInfo.InvariantCulture));
    }

    public void GetScoresSector(int sectorIndex, ScoreSectorCallback scoreCallback, StatusCallbackSector statusCallback = null)
    {
        scoreoidManager.SendRequest("getBestScores",
                                    delegate(bool success, string result, string error)
                                        {
                                            if (statusCallback != null)
                                                statusCallback(success, sectorIndex);

                                            if (success)
                                            {
                                                var scoresXml = new XmlDocument();
                                                scoresXml.LoadXml(result);
                                                var scores = ReadScores(scoresXml);
                                                scoreCallback(sectorIndex, scores);
                                            }
                                        },
                                    "response", "xml",
                                    "order_by", "score",
                                    "order", "desc",
                                    "difficulty", SectorIndexToDifficultyIndex(sectorIndex).ToString(CultureInfo.InvariantCulture));
    }

    public void GetScoresLevel(int levelScoreId, ScoreLevelCallback scoreCallback, StatusCallbackLevel statusCallback = null)
    {
        scoreoidManager.SendRequest("getBestScores",
                                    delegate(bool success, string result, string error)
                                        {
                                            if (statusCallback != null)
                                                statusCallback(success, levelScoreId);

                                            if (success)
                                            {
                                                var scoresXml = new XmlDocument();
                                                scoresXml.LoadXml(result);
                                                var scores = ReadScores(scoresXml);
                                                scoreCallback(levelScoreId, scores);
                                            }
                                        },
                                    "response", "xml",
                                    "order_by", "score",
                                    "order", "desc",
                                    "difficulty", levelScoreId.ToString(CultureInfo.InvariantCulture));
    }

    private List<ScoreEntry> ReadScores(XmlDocument scoresXml)
    {
        var root = scoresXml["scores"];
        var scoreNodeList = root.ChildNodes;

        int previousScore = int.MinValue;
        int placing = 0;
        int totalPlacing = 0;

        //Debug.Log("Parsing: " + scoresXml.InnerXml);
        var scores = new List<ScoreEntry>();
        foreach (XmlNode playerNode in scoreNodeList)
        {
            var scoreNode = playerNode.FirstChild;

            var username = playerNode.GetAttributeString("username");
            var score = scoreNode.GetAttributeInt("score");
            var dateString = scoreNode.GetAttributeString("created");
            var date = DateTime.Parse(dateString, DateTimeFormatInfo.InvariantInfo);

            totalPlacing++;
            if (score != previousScore)
            {
                previousScore = score;
                placing = totalPlacing;
            }

            scores.Add(new ScoreEntry(placing, username, score, date));
        }

        return scores;
    }

    private int SectorIndexToDifficultyIndex(int sectorIndex)
    {
        return (sectorIndex + 1) * 1000;
    }

    public int CalculateNewPlacing(int score, List<ScoreEntry> scoreEntries)
    {
        if (scoreEntries.Count == 0)
            return 1;

        /*
        Debug.Log("Score: " + score);
        Debug.Log("... in");
        foreach (var entry in scoreEntries)
        {
            Debug.Log(String.Format("  > {0}: {1} ({2}), {3}", entry.Placing, entry.Username, entry.Score, entry.Date));
        }
         */

        foreach (var entry in scoreEntries)
        {
            if (score >= entry.Score)
                return entry.Placing;
        }

        return scoreEntries.Last().Placing + 1;
    }
}
