﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class SplinePath
{
    private const int RandomPointTriesOuterLoop = 50;
    private const int RandomPointTriesInnerLoop = 30;
    private const int RandomPointNumberOfFollowUpPoints = 5;
    private const int RandomPointNumberOfFollowUpPointsTriesPerPoint = 10;

    private RageCurve spline;
    private int density;
    private int pointDistanceToEnd;

    private List<Vector2> splinePoints;
    private float tightness;

    //private int addedSplinePoints;
    //private float previousEnd;

    public SplinePath(RageCurve spline, int density, int pointDistanceToEnd, float tightness)
    {
        this.spline = spline;
        this.density = density;
        this.pointDistanceToEnd = pointDistanceToEnd;
        this.tightness = tightness;

        splinePoints = new List<Vector2>();
        UpdateSplinePoints();
    }

    private void UpdateSplinePoints()
    {
        splinePoints = new List<Vector2>();
        RefreshPoints();
    }

    public void RefreshPoints()
    {
        splinePoints.Clear();

        var pointCount = spline.points.Length;

        float offset = 1f / (pointCount * density);
        int count = (pointCount - 1 - pointDistanceToEnd) * density;
        for (int i = 0; i < count; i++)
        {
            var point = spline.GetPoint(offset * i);
            splinePoints.Add(point);
        }
    }

    /*
    private void AddNewPoints()
    {
        var pointCount = spline.points.Length;

        previousEnd *= pointCount;

        float offset = 1f / (pointCount * density);
        int count = (pointCount - 2) * density - addedSplinePoints;
        for (int i = 0; i < count; i++)
        {
            var point = spline.GetPoint(previousEnd + offset * i);
            splinePoints.Add(point);
        }
        addedSplinePoints += count;
        previousEnd += count * offset;

        previousEnd /= pointCount;
    }
     */

    public Vector2 GetSplinePositionBySplineIndex(float splineIndex)
    {
        while (splineIndex < 0f)
            splineIndex += splinePoints.Count;

        var startIndex = (int) splineIndex;
        var startPoint = GetSplinePoint(startIndex);

        if (startIndex == splineIndex)
            return startPoint;

        var endPoint = GetSplinePoint(startIndex + 1);

        var t = splineIndex - startIndex;

        return Vector2.Lerp(startPoint, endPoint, t);
    }

    public Vector2 GetSplinePoint(int index)
    {
        return splinePoints[index % splinePoints.Count];
    }

    public float MoveSplineIndex(float splineIndex, float distance, int moveDirection)
    {
        if (distance <= 0)
            return splineIndex;

        if ((splineIndex >= (splinePoints.Count - 1)) && (moveDirection > 0))
            return splinePoints.Count - 1;

        if ((splineIndex <= 0) && (moveDirection < 0))
            return 0;

        var centerPosition = GetSplinePositionBySplineIndex(splineIndex);

        var currentStartIndex = (int) splineIndex;

        // Is the point we search for at the current vertex?
        var fromPointStart = splinePoints[currentStartIndex];
        var toPointStart = splinePoints[currentStartIndex + 1];
        var totalDistanceStart = Vector2.Distance(fromPointStart, toPointStart);
        if (((moveDirection > 0) && (distance <= Vector2.Distance(centerPosition, toPointStart))) ||
            ((moveDirection < 0) && (distance <= Vector2.Distance(centerPosition, fromPointStart))))
        {
            return splineIndex + distance / totalDistanceStart * moveDirection;
        }

        do
        {
            currentStartIndex += moveDirection;

            if ((currentStartIndex >= (splinePoints.Count - 1)) && (moveDirection > 0))
                return splinePoints.Count - 1;

            if ((currentStartIndex < 0) && (moveDirection < 0))
                return 0;

            var fromPoint = splinePoints[currentStartIndex];
            var toPoint = splinePoints[currentStartIndex + 1];
            var intersections = MathUtil.CircleIntersections(centerPosition, distance, fromPoint, toPoint, true);

            if (intersections != null)
            {
                if (moveDirection < 0)
                    GeneralHelper.Swap(ref fromPoint, ref toPoint);

                Vector2 intersection;
                if (intersections.Count == 1)
                {
                    intersection = intersections[0];
                }
                else
                {
                    var distance1 = Vector2.Distance(toPoint, intersections[0]);
                    var distance2 = Vector2.Distance(toPoint, intersections[1]);
                    intersection = (distance1 <= distance2) ? intersections[0] : intersections[1];
                }

                var totalDistance = Vector2.Distance(fromPoint, toPoint);
                var intersectionDistance = Vector2.Distance(fromPoint, intersection);

                var t = intersectionDistance / totalDistance;

                if (moveDirection > 0)
                {
                    return currentStartIndex + t;
                }
                else
                {
                    return currentStartIndex + 1 - t;
                }
            }
        } while (true);
    }

    public float RemoveUnusedPoints(float minimalNeededPosition)
    {
        if (minimalNeededPosition <= density)
            return minimalNeededPosition;

        while (minimalNeededPosition > density)
        {
            /*
            previousEnd *= spline.points.Length;
            previousEnd -= (spline.points.Length - 2) * density - addedSplinePoints;
            previousEnd /= spline.points.Length;
            addedSplinePoints -= density;
             */

            minimalNeededPosition -= density;
            spline.DelPoint(0);
            splinePoints.RemoveRange(0, density);
        }

        return minimalNeededPosition;
    }

    public int PointCount
    {
        get { return splinePoints.Count; }
    }

    public void RefreshControlPoints()
    {
        var controlPoints = FindCPoints(spline.points.Select(rageSplinePoint => (Vector2) rageSplinePoint.point).ToArray());
        for (int i = 0; i < spline.points.Length; i++)
        {
            var rageSplinePoint = spline.points[i];
            rageSplinePoint.inCtrl = -controlPoints[i];
            rageSplinePoint.outCtrl = controlPoints[i];
        }

        RefreshPoints();
    }

    // http://www.ibiblio.org/e-notes/Splines/Bint.java
    public Vector2[] FindCPoints(Vector2[] p)
    {
        int n = p.Length;

        var d = new Vector2[n];
        for (int i = 0; i < d.Length; i++)
            d[i] = new Vector2();

        var bI = new float[n];
        var aX = new float[n];
        var aY = new float[n];

        bI[1] = -tightness;
        aX[1] = (p[2].x - p[0].x - d[0].x) / 4;
        aY[1] = (p[2].y - p[0].y - d[0].y) / 4;

        for (int i = 2; i < n - 1; i++)
        {
            bI[i] = -1 / (4 + bI[i - 1]);
            aX[i] = -(p[i + 1].x - p[i - 1].x - aX[i - 1]) * bI[i];
            aY[i] = -(p[i + 1].y - p[i - 1].y - aY[i - 1]) * bI[i];
        }

        for (int i = n - 2; i > 0; i--)
        {
            d[i].x = aX[i] + d[i + 1].x * bI[i];
            d[i].y = aY[i] + d[i + 1].y * bI[i];
        }

        return d;
    }

    public void AddRandomPoint(float minimalDistance, float maximalDistance, float maxAngleDelta, Rect area)
    {
        var border = maximalDistance * tightness;
        var innerLeft = area.xMin + border;
        var innerTop = area.yMin + border;
        var innerRight = area.xMax - border;
        var innerBottom = area.yMax - border;

        Vector2 newPoint;

        var existingPoints = spline.points.Select(rageSplinePoint => (Vector2) rageSplinePoint.point).ToArray();

        if (existingPoints.Length == 0)
        {
            // points.length == 0 --> create first point

            newPoint = area.RandomPosition(border);
        }
        else
        {
            // points.length > 0

            Vector2 lastPoint = existingPoints[existingPoints.Length - 1];
            float newPointAngle;

            int triesTotal = RandomPointTriesOuterLoop;
            do
            {
                if (existingPoints.Length == 1)
                {

                    int triesInner = RandomPointTriesInnerLoop;
                    // points.length == 1 --> create second point near (minimalDistance, maximalDistance) first point inside boundaries
                    do
                    {
                        newPointAngle = UnityEngine.Random.value * 360f;
                        float distance = UnityEngine.Random.Range(minimalDistance, maximalDistance);
                        newPoint = lastPoint + UnityHelper.CreateVector2AngleDeg(newPointAngle) * distance;

                        triesInner--;
                        if (triesInner < 0)
                        {
                            Debug.Log("triesInner exhausted; aborting");
                            break;
                        }

                    } while ((newPoint.x < innerLeft) || (newPoint.x > innerRight) || (newPoint.y < innerTop) || (newPoint.y > innerBottom));
                }
                else
                {
                    // point.length > 1 --> create next point according to distance and angle inside boundaries

                    var nextToLastPoint = existingPoints[existingPoints.Length - 2];

                    double previousAngle = (lastPoint - nextToLastPoint).GetAngleDeg();

                    int triesInner = RandomPointTriesInnerLoop;

                    // Find a point between minDistance and maxDistance with an angle between previousAngle+/-angleRangeToEachSide
                    // that is inside the boundaries
                    do
                    {
                        newPointAngle = (float) (previousAngle + UnityEngine.Random.Range(-maxAngleDelta, maxAngleDelta));
                        float distance = UnityEngine.Random.Range(minimalDistance, maximalDistance);
                        newPoint = lastPoint + UnityHelper.CreateVector2AngleDeg(newPointAngle) * distance;

                        triesInner--;
                        if (triesInner < 0)
                        {
                            Debug.Log("triesInner exhausted; aborting");
                            break;
                        }

                    } while ((newPoint.x < innerLeft) || (newPoint.x > innerRight) || (newPoint.y < innerTop) || (newPoint.y > innerBottom));

                }

                triesTotal--;
                if (triesTotal < 0)
                {
                    Debug.Log("triesTotal exhausted; aborting");
                    break;
                }

                // Check if you can find a path of a few point from that new point. If you cannot, this is a dead end and we need another next point
            } while (!FindFollowupRandomPoints(RandomPointNumberOfFollowUpPoints, RandomPointNumberOfFollowUpPointsTriesPerPoint, newPointAngle, newPoint, maxAngleDelta, minimalDistance,
                                               maximalDistance, innerLeft, innerTop, innerRight, innerBottom));
        }

        spline.AddRageSplinePoint(spline.points.Length, newPoint, Vector3.zero, Vector3.zero, 1f, true);
    }

    private bool FindFollowupRandomPoints(int numberOfFollowupPoints, int numberOfTriesEachPoint, float newPointAngle, Vector2 lastPoint, float angleRangeToEachSide,
                                          float minimalDistance, float maximalDistance, float innerLeft, float innerTop, float innerRight, float innerBottom)
    {
        // If there are 0 points left, we abort
        if (numberOfFollowupPoints == 0)
        {
            return true;
        }

        for (int remainingTrys = numberOfTriesEachPoint; remainingTrys > 0; remainingTrys--)
        {
            float angle = (float) (newPointAngle + UnityEngine.Random.Range(-angleRangeToEachSide, angleRangeToEachSide));
            float distance = UnityEngine.Random.Range(minimalDistance, maximalDistance);

            Vector2 tryPoint = lastPoint + UnityHelper.CreateVector2AngleDeg(angle) * distance;
            if ((tryPoint.x >= innerLeft) && (tryPoint.x <= innerRight) && (tryPoint.y >= innerTop) && (tryPoint.y <= innerBottom))
            {
                return FindFollowupRandomPoints(numberOfFollowupPoints - 1, numberOfTriesEachPoint, angle, tryPoint, angleRangeToEachSide,
                                                minimalDistance, maximalDistance, innerLeft, innerTop, innerRight, innerBottom);
            }
        }
        return false;
    }
}
