﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

#if UNITY_EDITOR
using UnityEditor;
#endif

public abstract class AbstractRageShape : MonoBehaviour
{
    public delegate void RefreshDelegate(bool pointsChanged);
    public delegate void RefreshColorDelegate(Color? outlineColor, Color? fillColor, Color? outlineColorGlow);
    public delegate void RefreshColorAlphaDelegate(float percent);

    [SerializeField] private float outlineWidth;
    [SerializeField] private float antiAliasWidth;
    [SerializeField] protected float scale = 1f;
    [SerializeField] private bool glow = false;
    [SerializeField] private bool ignoreShape = false;
    //[SerializeField] private float colliderSize = 0f;

    [SerializeField] private int vertexDensity = 0;

    protected VisualSettings visualSettings;
    protected RageSpline rageSpline;

    private bool pointsChanged;

    private bool materialCreated;

    [SerializeField] [HideInInspector] private Color defaultOutlineColor;
    [SerializeField] [HideInInspector] private Color defaultFillColor;
    [SerializeField] [HideInInspector] private bool colorsSet;

    public Color DefaultOutlineColor { get { return defaultOutlineColor; } private set { defaultOutlineColor = value; } }
    public Color DefaultFillColor { get { return defaultFillColor; } private set { defaultFillColor = value; } }

    public float BoundingRadius { get; private set; }

    public event RefreshDelegate EventRefreshed;
    public event RefreshColorDelegate EventRefreshedColor;
    public event RefreshColorAlphaDelegate EventRefreshedColorAlpha;

    protected virtual void Awake()
    {
        visualSettings = VisualSettings.Instance;
        rageSpline = gameObject.GetComponent<RageSpline>();

        Refresh();

        // Save the default colors on first Awake - but don't get them again if this is copied via Instantiate
        if (!colorsSet)
        {
            DefaultOutlineColor = rageSpline.GetOutlineColor1();
            DefaultFillColor = rageSpline.GetFillColor1();
            colorsSet = true;
        }
    }

    protected virtual void OnDestroy()
    {
        if (materialCreated)
        {
            Destroy(renderer.material);
        }
    }

    /*
    private void OnReplicated(GameObject original)
    {
        var originalAbstractShape = original.GetComponent<AbstractShape>();
        DefaultOutlineColor = originalAbstractShape.DefaultOutlineColor;
        DefaultFillColor = originalAbstractShape.DefaultFillColor;
        rageSpline.RefreshMesh(false, false, false);
    }
     */

    protected virtual void Start()
    {
    }

    public virtual void Refresh()
    {
        if (visualSettings == null)
            visualSettings = VisualSettings.Instance;

        if (rageSpline == null)
            rageSpline = gameObject.GetComponent<RageSpline>();

        RefreshVisualSettings();
#if UNITY_EDITOR
        EditorUtility.SetDirty(rageSpline);
#endif
    }

    private void RefreshVisualSettings()
    {
        /*
        if (fillColor.a == 0)
        {
            rageSpline.SetFill(RageSpline.Fill.None);
        }
        else
        {
            rageSpline.SetFill(RageSpline.Fill.Solid);
            rageSpline.SetFillColor1(fillColor);
        }
         */

        /*
        if (collidable)
        {
            rageSpline.SetPhysics(RageSpline.Physics.MeshCollider);
            rageSpline.SetPhysicsZDepth(0.5f);
            rageSpline.SetCreatePhysicsInEditor(true);
            rageSpline.PhysicsIsTrigger = true;
        }
        else
        {
            rageSpline.SetPhysics(RageSpline.Physics.None);
        }
         */
        rageSpline.SetPhysics(RageSpline.Physics.None);

        rageSpline.SetOutlineWidth((outlineWidth > 0) ? outlineWidth : visualSettings.OutlineWidth);
        rageSpline.SetAntialiasingWidth((antiAliasWidth > 0) ? antiAliasWidth: visualSettings.AntiAliasWidth);
    }

    /*
    protected void RefreshColliderSize(float scale)
    {
        var sphereCollider = GetComponent<SphereCollider>();

        var radius = scale * colliderSize / 2f;
        if (radius <= 0)
        {
            if (sphereCollider != null)
                DestroyImmediate(sphereCollider);

            return;
        }

        if (sphereCollider == null)
        {
            sphereCollider = gameObject.AddComponent<SphereCollider>();
            sphereCollider.isTrigger = true;
        }

        if (sphereCollider.radius == radius)
            return;

        sphereCollider.radius = radius;
    }
     */

    public void RefreshPointCount(int targetCount)
    {
        if (targetCount < 2)
            throw new ArgumentException("You can't have a shape with less than 2 points.");

        if (rageSpline.GetPointCount() == targetCount)
            return;

        while (rageSpline.GetPointCount() > targetCount)
        {
            rageSpline.RemovePoint(rageSpline.GetPointCount() - 1);
        }

        while (rageSpline.GetPointCount() < targetCount)
        {
            rageSpline.AddPoint(rageSpline.GetPointCount(), Vector3.zero);
        }

        pointsChanged = true;
    }

    protected void SetNaturalPoint(int i, Vector3 position, Vector3 innerControlPoint, float widthMultiplier = 1)
    {
        rageSpline.SetPoint(i, position, innerControlPoint, -innerControlPoint, widthMultiplier, true);

        pointsChanged = true;
    }

    protected virtual void RefreshMesh()
    {
        if (vertexDensity > 0)
            rageSpline.VertexDensity = vertexDensity;

        //var isFilled = rageSpline.GetFill() != RageSpline.Fill.None;
        if (Application.isPlaying)
        {
            rageSpline.RefreshMesh(true, true, true);
        }
        else
        {
            rageSpline.RefreshMeshInEditor(true, true, true);
        }

        RefreshRadius();

        if (EventRefreshed != null)
            EventRefreshed(pointsChanged);

        pointsChanged = false;
    }

    public void RefreshRadius()
    {
        var radiusSqr = 0f;
        for (int i = 0; i < rageSpline.GetPointCount(); i++)
        {
            var magnitudeSq = rageSpline.GetPosition(i).sqrMagnitude;
            if (magnitudeSq > radiusSqr)
                radiusSqr = magnitudeSq;
        }

        BoundingRadius = Mathf.Sqrt(radiusSqr);
    }

    public Vector3 GetPointWorldPosition(int pointIndex)
    {
        return rageSpline.GetPositionWorldSpace(pointIndex);
    }

    public int PointCount { get { return rageSpline.GetPointCount(); } }

    public bool CheckCollision(Vector2 previousWorldPosition, Vector2 currentWorldPosition)
    {
        var points = new Vector2[rageSpline.GetPointCount()];
        for (int i = 0; i < points.Length; i++)
        {
            points[i] = rageSpline.GetPositionWorldSpace(i);
        }
        if (MathUtil.PolygonContainsOrTouches(points, currentWorldPosition))
            return true;

        if (previousWorldPosition != currentWorldPosition)
        {
            if (MathUtil.CheckPolygonLineIntersection(points, previousWorldPosition, currentWorldPosition))
                return true;
        }

        return false;
    }

    /*
    public void DebugCheckCollision(Vector3 worldPosition)
    {
        Debug.ClearDeveloperConsole();
        var str = "Collision with " + gameObject.name + ": [";
        var points = new Vector2[rageSpline.GetPointCount()];
        for (int i = 0; i < points.Length; i++)
        {
            points[i] = rageSpline.GetPositionWorldSpace(i);
            str += points[i] + " ";
        }
        str += "] with " + worldPosition + ": ";
        str += MathUtil.PolygonContainsOrTouches(points, worldPosition);

        Debug.Log(str, gameObject);
        Debug.Break();
    }
     */

    public void RefreshColors(Color? outlineColor, Color? fillColor, Color? outlineColorGlow)
    {
        rageSpline.SetOutlineColor1(outlineColor ?? DefaultOutlineColor);
        rageSpline.SetFillColor1(fillColor ?? DefaultFillColor);
        rageSpline.RefreshMesh(false, false, false);

        if (EventRefreshedColor != null)
            EventRefreshedColor(outlineColor, fillColor, outlineColorGlow);
    }

    public void RefreshColors(float percent)
    {
        /*
        var outlineColor = DefaultOutlineColor;
        outlineColor.a *= percent;

        var fillColor = DefaultFillColor;
        fillColor.a *= percent;

        rageSpline.SetOutlineColor1(outlineColor);
        rageSpline.SetFillColor1(fillColor);
        rageSpline.RefreshMesh(false, false, false);
         */

        renderer.material.color = new Color(1f, 1f, 1f, percent);
        materialCreated = true;

        if (EventRefreshedColorAlpha != null)
            EventRefreshedColorAlpha(percent);
    }

    public float Scale
    {
        get { return scale; }
        set { scale = value; }
    }

    public bool Glow { get { return glow; } }
    public bool IgnoreShape { get { return ignoreShape; } }
}
