﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

#if UNITY_EDITOR
using UnityEditor;
#endif

[ExecuteInEditMode]
[RequireComponent(typeof(RageSpline))]
public class RageGlowCopyParentShape : MonoBehaviour
{
    [SerializeField] private float outlineWidth;
    [SerializeField] private float antiAliasWidth;
    [SerializeField] private float scale = 1f;
    [SerializeField] private float scaleControlPoints = 1f;
    [SerializeField] private int vertexDensity = 0;

    protected RageSpline rageSpline;
    protected AbstractRageShape parentShape;
    protected RageSpline parentRageSpline;

    private VisualSettings visualSettings;
    private bool materialCreated;

    [SerializeField] [HideInInspector] private Color defaultOutlineColor;
    [SerializeField] [HideInInspector] private Color defaultFillColor;
    [SerializeField] [HideInInspector] private bool colorsSet;

    public Color DefaultOutlineColor { get { return defaultOutlineColor; } private set { defaultOutlineColor = value; } }
    public Color DefaultFillColor { get { return defaultFillColor; } private set { defaultFillColor = value; } }

    protected void Awake()
    {
        rageSpline = gameObject.GetComponent<RageSpline>();
        parentShape = transform.parent.GetComponent<AbstractRageShape>();
        parentRageSpline = parentShape.GetComponent<RageSpline>();

        visualSettings = VisualSettings.Instance;

        // Save the default colors on first Awake - but don't get them again if this is copied via Instantiate
        if (!colorsSet)
        {
            DefaultOutlineColor = rageSpline.GetOutlineColor1();
            DefaultFillColor = rageSpline.GetFillColor1();
            colorsSet = true;
        }
        
        transform.localPosition = new Vector3(transform.localPosition.x, transform.localPosition.y, 1);
    }

    protected virtual void OnDestroy()
    {
        if (materialCreated)
        {
            Destroy(renderer.material);
        }
    }

    /*
    private void OnReplicated(GameObject original)
    {
        var originalGlowCopyParentShape = original.GetComponent<GlowCopyParentShape>();
        DefaultOutlineColor = originalGlowCopyParentShape.DefaultOutlineColor;
        DefaultFillColor = originalGlowCopyParentShape.DefaultFillColor;
        rageSpline.RefreshMesh(false, false, false);
    }
     */

    protected void OnEnable()
    {
        parentShape = transform.parent.GetComponent<AbstractRageShape>();
        parentRageSpline = parentShape.GetComponent<RageSpline>();

        Refresh(true);
        parentShape.EventRefreshed += OnParentShapeRefreshed;
        parentShape.EventRefreshedColor += OnParentShapeRefreshedColor;
        parentShape.EventRefreshedColorAlpha += OnParentShapeRefreshedColorAlpha;
    }

    protected void OnDisable()
    {
        parentShape = transform.parent.GetComponent<AbstractRageShape>();
        parentRageSpline = parentShape.GetComponent<RageSpline>();

        parentShape.EventRefreshed -= OnParentShapeRefreshed;
        parentShape.EventRefreshedColor -= OnParentShapeRefreshedColor;
        parentShape.EventRefreshedColorAlpha -= OnParentShapeRefreshedColorAlpha;
    }

    private void OnParentShapeRefreshed(bool pointsChanged)
    {
        Refresh(true);
    }

    private void OnParentShapeRefreshedColor(Color? outlineColor, Color? fillColor, Color? outlineColorGlow)
    {
        RefreshColors(outlineColorGlow, null);
    }

    private void OnParentShapeRefreshedColorAlpha(float percent)
    {
        RefreshColors(percent);
    }

    public void Refresh(bool refreshPoints)
    {
        renderer.enabled = parentShape.Glow;
        rageSpline.enabled = parentShape.Glow;

        if (!parentShape.Glow)
            return;

        if (refreshPoints)
            RefreshPoints();

        RefreshMesh();

#if UNITY_EDITOR
        EditorUtility.SetDirty(rageSpline);
#endif
    }

    private void RefreshPoints()
    {
        var parentPointCount = parentRageSpline.GetPointCount();

        RefreshPointCount(parentPointCount);

        for (int i = 0; i < parentPointCount; i++)
        {
            rageSpline.SetPoint(i,
                                parentRageSpline.GetPosition(i) * scale,
                                parentRageSpline.GetInControlPositionPointSpace(i) * scaleControlPoints,
                                parentRageSpline.GetOutControlPositionPointSpace(i) * scaleControlPoints,
                                parentRageSpline.GetOutlineWidth(i),
                                parentRageSpline.GetNatural(i));
        }
    }

    public void RefreshPointCount(int targetCount)
    {
        if (targetCount < 2)
            throw new ArgumentException("You can't have a shape with less than 2 points.");

        if (rageSpline.GetPointCount() == targetCount)
            return;

        while (rageSpline.GetPointCount() > targetCount)
        {
            rageSpline.RemovePoint(rageSpline.GetPointCount() - 1);
        }

        while (rageSpline.GetPointCount() < targetCount)
        {
            rageSpline.AddPoint(rageSpline.GetPointCount(), Vector3.zero);
        }
    }

    protected void RefreshMesh()
    {
        rageSpline.SetPhysics(RageSpline.Physics.None);

        rageSpline.SetOutlineWidth((outlineWidth > 0) ? outlineWidth : visualSettings.OutlineWidth);
        rageSpline.SetAntialiasingWidth((antiAliasWidth > 0) ? antiAliasWidth: visualSettings.AntiAliasWidth);

        rageSpline.VertexDensity = (vertexDensity > 0) ? vertexDensity : parentRageSpline.VertexDensity;

        if (Application.isPlaying)
        {
            rageSpline.RefreshMesh(true, true, true);
        }
        else
        {
            rageSpline.RefreshMeshInEditor(true, true, true);
        }
    }

    public void RefreshColors(Color? outlineColor, Color? fillColor)
    {
        fillColor = (rageSpline.GetFill() != RageSpline.Fill.None) ? outlineColor : null;

        rageSpline.SetOutlineColor1(outlineColor ?? DefaultOutlineColor);
        rageSpline.SetFillColor1(fillColor ?? DefaultFillColor);
        rageSpline.RefreshMesh(false, false, false);
    }

    public void RefreshColors(float percent)
    {
        /*
        var outlineColor = DefaultOutlineColor;
        outlineColor.a *= percent;

        var fillColor = DefaultFillColor;
        fillColor.a *= percent;

        RefreshColors(outlineColor, fillColor);
         */

        renderer.material.color = new Color(1f, 1f, 1f, percent);
        materialCreated = true;
    }

    public float Scale
    {
        get { return scale; }
    }

    public float ScaleControlPoints
    {
        get { return scaleControlPoints; }
    }

    public int VertexDensity
    {
        get { return vertexDensity; }
        set { vertexDensity = value; }
    }
}
